@@ -24,7 +24,11 @@
 /* FIXME: Should use BonoboUIHandler.  */
 
 #include <config.h>
+#ifdef GNOME_GTKHTML_EDITOR_SHLIB
+#include <glib/gi18n-lib.h>
+#else
 #include <glib/gi18n.h>
+#endif
 #include <gnome.h>
 #include <bonobo.h>
 
@@ -286,6 +290,13 @@ load_done (GtkHTML *html, GtkHTMLControlData *cd)
 		g_signal_connect (cd->html, "realize", G_CALLBACK (realize_engine), cd);
 }
 
+static void
+insertion_color_changed_cb (GtkHTML *widget, GdkColor *color, GtkHTMLControlData *cd)
+{
+	gi_color_combo_set_color ((GiColorCombo *) cd->combo, color);
+}
+
+
 static GtkWidget *
 setup_gi_color_combo (GtkHTMLControlData *cd)
 {
@@ -299,7 +310,8 @@ setup_gi_color_combo (GtkHTMLControlData *cd)
         g_signal_connect (cd->html, "load_done", G_CALLBACK (load_done), cd);
 
 	cd->combo = gi_color_combo_new (NULL, _("Automatic"), &color->color, color_group_fetch ("toolbar_text", cd));
-        g_signal_connect (cd->combo, "color_changed", G_CALLBACK (color_changed), cd);
+	g_signal_connect (cd->combo, "color_changed", G_CALLBACK (color_changed), cd);
+	g_signal_connect (cd->html, "insertion_color_changed", G_CALLBACK (insertion_color_changed_cb), cd);
 
 	gtk_widget_show_all (cd->combo);
 	return cd->combo;
