@@ -174,22 +174,24 @@ glyphs_destroy (GList *glyphs)
 }
 
 inline static void
-glyph_items_destroy (GList *glyph_items)
+glyph_items_destroy (GSList *glyph_items)
 {
-	GList *l;
+	GSList *l;
 
-	for (l = glyph_items; l; l = l->next->next) {
-		HTMLTextSlaveGlyphItem *gi = (HTMLTextSlaveGlyphItem *) gi;
+	for (l = glyph_items; l; l = l->next) {
+		HTMLTextSlaveGlyphItem *gi = (HTMLTextSlaveGlyphItem *) l->data;
 
 		if (gi->type == HTML_TEXT_SLAVE_GLYPH_ITEM_CREATED) {
-			pango_item_free (gi->glyph_item.item);
-			pango_glyph_string_free (gi->glyph_item.glyphs);
+			if (gi->glyph_item.item)
+				pango_item_free (gi->glyph_item.item);
+			if (gi->glyph_item.glyphs)
+				pango_glyph_string_free (gi->glyph_item.glyphs);
 			g_free (gi->widths);
 		}
 
 		g_free (gi);
 	}
-	g_list_free (glyph_items);
+	g_slist_free (glyph_items);
 }
 
 static gboolean
@@ -644,8 +646,13 @@ get_glyph_items_in_range (HTMLTextSlave *slave, HTMLPainter *painter, int start_
 		offset += item->num_chars;
 	}
 
-	if (glyph_items)
-		glyph_items = reorder_glyph_items (g_slist_reverse (glyph_items), n_items);
+	if (glyph_items) {
+		GSList *reversed;
+
+		reversed = g_slist_reverse (glyph_items);
+		glyph_items = reorder_glyph_items (reversed, n_items);
+		g_slist_free (reversed);
+	}
 
 	return glyph_items;
 }
@@ -747,7 +754,7 @@ draw_text (HTMLTextSlave *self,
 	selection = isect_start < isect_end;
 
 	if (p->widget && GTK_IS_HTML (p->widget))
-		e = GTK_HTML (p->widget)->engine;
+		e = html_object_engine (HTML_OBJECT (self->owner), GTK_HTML (p->widget)->engine);
 
 	if (selection) {
 		gchar *end;
@@ -777,6 +784,9 @@ draw_text (HTMLTextSlave *self,
 		GList *cur_se;
 		int cur_width;
 
+		if (e)
+			html_painter_set_pen (p, &html_colorset_get_color_allocated (e->settings->color_set,
+										     e->painter, HTMLTextColor)->color);
 		cur_width = html_painter_draw_glyphs (p, obj->x + tx + html_painter_pango_to_engine (p, run_width),
 						      obj->y + ty + get_ys (text, p), gi->glyph_item.item, gi->glyph_item.glyphs, NULL, NULL);
 
@@ -841,7 +851,7 @@ draw_text (HTMLTextSlave *self,
 }
 
 static void
-draw_focus_rectangle  (HTMLPainter *painter, GdkRectangle *box)
+draw_focus_rectangle  (HTMLTextSlave *slave, HTMLPainter *painter, GdkRectangle *box)
 {
 	HTMLGdkPainter *p;
 	GdkGCValues values;
@@ -849,7 +859,7 @@ draw_focus_rectangle  (HTMLPainter *painter, GdkRectangle *box)
 	HTMLEngine *e;
 
 	if (painter->widget && GTK_IS_HTML (painter->widget))
-		e = GTK_HTML (painter->widget)->engine;
+		e = html_object_engine (HTML_OBJECT (slave->owner), GTK_HTML (painter->widget)->engine);
 	else
 		return;
 
@@ -889,7 +899,7 @@ draw_focus (HTMLTextSlave *slave, HTMLPainter *p, gint tx, gint ty)
 			rect.width -= html_text_calc_part_width (slave->owner, p, slave->owner->text + link->end_index, link->end_offset,
 								 slave->posStart + slave->posLen - link->end_offset, NULL, NULL);
 		rect.y += ty;
-		draw_focus_rectangle (p, &rect);
+		draw_focus_rectangle (slave, p, &rect);
 	}
 }
 
@@ -1042,21 +1052,12 @@ check_point (HTMLObject *self,
 	return NULL;
 }
 
-static void
-clear_glyphs (HTMLTextSlave *slave)
-{
-	if (slave->glyphs) {
-		glyphs_destroy (slave->glyphs);
-		slave->glyphs = NULL;
-	}
-}
-
 static void
 clear_glyph_items (HTMLTextSlave *slave)
 {
 	if (slave->glyph_items) {
-		glyph_items_destroy (slave->glyphs);
-		slave->glyphs = NULL;
+		glyph_items_destroy (slave->glyph_items);
+		slave->glyph_items = NULL;
 	}
 }
 
@@ -1065,7 +1066,7 @@ destroy (HTMLObject *obj)
 {
 	HTMLTextSlave *slave = HTML_TEXT_SLAVE (obj);
 
-	clear_glyphs (slave);
+	clear_glyph_items (slave);
 
 	HTML_OBJECT_CLASS (parent_class)->destroy (obj);
 }
@@ -1125,7 +1126,6 @@ html_text_slave_init (HTMLTextSlave *slave,
 	slave->owner      = owner;
 	slave->charStart  = NULL;
 	slave->pi         = NULL;
-	slave->glyphs     = NULL;
 	slave->glyph_items = NULL;
 
 	/* text slaves have always min_width 0 */
@@ -1175,6 +1175,8 @@ html_text_slave_get_glyph_item_at_offset (HTMLTextSlave *slave, HTMLPainter *pai
 	GSList *cur;
 	int index;
 
+	prev_gi = next_gi = NULL;
+
 	index = g_utf8_offset_to_pointer (html_text_slave_get_text (slave), offset) - slave->owner->text;
 	if (index_out)
 		*index_out = index;
@@ -1274,7 +1276,7 @@ html_text_slave_cursor_right_one (HTMLTextSlave *slave, HTMLPainter *painter, HT
 		}
 	} else {
 		/* RTL */
-		if (index > gi->glyph_item.item->offset && (index <= gi->glyph_item.item->offset + gi->glyph_item.item->length)) {
+		if (index > gi->glyph_item.item->offset && index <= gi->glyph_item.item->offset + gi->glyph_item.item->length) {
 			cursor->offset --;
 			cursor->position --;
 
@@ -1319,6 +1321,7 @@ html_text_slave_cursor_left_one (HTMLTextSlave *slave, HTMLPainter *painter, HTM
 {
 	HTMLTextSlaveGlyphItem *prev, *next;
 	int index;
+	HTMLObject *prev_obj = HTML_OBJECT (slave->owner)->prev;
 	HTMLTextSlaveGlyphItem *gi = html_text_slave_get_glyph_item_at_offset (slave, painter, cursor->offset - slave->posStart, &prev, &next, NULL, &index);
 
 /* 	printf ("gi: %p item num chars: %d\n", gi, gi ? gi->glyph_item.item->num_chars : -1); */
@@ -1328,7 +1331,7 @@ html_text_slave_cursor_left_one (HTMLTextSlave *slave, HTMLPainter *painter, HTM
 
 	if (gi->glyph_item.item->analysis.level % 2 == 0) {
 		/* LTR */
-		if (index - gi->glyph_item.item->offset > 1 || (!prev && index - gi->glyph_item.item->offset > 0)) {
+		if (index - gi->glyph_item.item->offset > 1 || (!prev && !prev_obj && index - gi->glyph_item.item->offset > 0)) {
 			cursor->offset --;
 			cursor->position --;
 
